
// ZIM LIBRARY TEMPLATE

// ~~~~~~~~~~~~~~~~~~~~~~~~
// DESCRIPTION 
// This library is an example library/module in ES6 module format 
// The code inside the module is ES6 
// We introduce ZIM VEE (Pick) and ZIM OCT (STYLE)
// Note the parameters in the docs change

/*--
House = function(size, color, topColor)

House
class - extends a zim.Container which extends a createjs.Container

DESCRIPTION
An example component that makes a simple house.

EXAMPLE
const house = new House(200,300,red,grey).center();
END EXAMPLE

PARAMETERS
** supports DUO - parameters or single object with properties below
** supports VEE - parameters marked with ZIM VEE mean a zim Pick() object or Pick Literal can be passed
   Pick Literal formats: [1,3,2] - random; {min:10, max:20} - range; series(1,2,3) - order, function(){return result;} - function
** supports OCT - parameter defaults can be set with STYLE control (like CSS)
size - (default 100) width and height if the bottom of the house
bottomColor - (default tin) color of the rectangle of the house
topColor - (default red) color of the triangle of the house
style - (default true) set to false to ignore styles set with the STYLE - will receive original parameter defaults
group - (default null) set to String (or comma delimited String) so STYLE can set default styles to the group(s) (like a CSS class)
inherit - (default null) used internally but can receive an {} of styles directly

METHODS
** these methods are available only once the WebCam "ready" event has happened.
fall(direction) - collapse the house - direction can be LEFT or RIGHT (default)
    returns obj for chaining
dispose() - nothing really to dispose here but will add the method for reference

ALSO: ZIM 4TH adds all the methods listed under Container (see above), such as:
drag(), hitTestRect(), animate(), sca(), reg(), mov(), center(), centerReg(),
addTo(), removeFrom(), loop(), outline(), place(), pos(), alp(), rot(), setMask(), etc.
ALSO: see the CreateJS Easel Docs for Container methods, such as:
on(), off(), getBounds(), setBounds(), cache(), uncache(), updateCache(), dispatchEvent(),
addChild(), removeChild(), addChildAt(), getChildAt(), contains(), removeAllChildren(), etc.

PROPERTIES
type - name of class as a string
bottom - reference to the rectangle at bottom 
top - reference to the triangle at top
bottomColor - |ZIM VEE| get or set bottom color
topColor - |ZIM VEE| get or set top color

ALSO: see ZIM Container for properties such as:
width, height, widthOnly, heightOnly, draggable, level, depth, group 
blendMode, hue, saturation, brightness, contrast, etc.

ALSO: see the CreateJS Easel Docs for Container properties, such as:
x, y, rotation, scaleX, scaleY, regX, regY, skewX, skewY,
alpha, cursor, shadow, name, mouseChildren, mouseEnabled, parent, numChildren, etc.

EVENTS
dispatches a "fallen" event when house has fallen

See the CreateJS Easel Docs for Stage events, such as:
mouseenter, mouseleave, stagemousedown, stagemousemove, stagemouseup, drawstart, drawend, etc.
and all the Container events such as:
click, dblclick, mousedown, mouseout, mouseover, pressdown (ZIM), pressmove, pressup, removed, rollout, rollover
*/

// we use zim.Container, zim.tin, zim.red, etc. in case this is used with ZIM with the ZIM namespace required

// ES6 Class
export class House extends zim.Container {        
    // cannot use ES6 default parameters with style as they rely on a variable
    constructor(size, bottomColor, topColor, style, group, inherit) {
        
        // NOTE: do not use ZIM DUO if passing parameters to the super
        super(); // super needs to be run before keyword this can be used
        // for ES6, a sig is required (as it would be if the file is minified)
        const sig = "size, bottomColor, topColor, style, group, inherit";
        let duo; if (duo = zob(House, arguments, sig, this)) return duo;        
        
        this.type = "House";

        // FOR STYLE
        this.group = group;
        let DS = style===false?{}:zim.getStyle(this.type, this.group, inherit);
    
        // READ IN STYLE 
        if (size==null) size = DS.size!=null?DS.size:100;
        if (bottomColor==null) bottomColor = DS.bottomColor!=null?DS.bottomColor:zim.tin;
        if (topColor==null) topColor = DS.topColor!=null?DS.topColor:zim.red;

        // PICK
        // pick fro ZIM VEE values
        size = zim.Pick.choose(size); // or zik(size)
        bottomColor = zim.Pick.choose(bottomColor);
        topColor = zim.Pick.choose(topColor);            
                
        // need to reset the size of the Container
        const width = size;
        const height = width+Math.sqrt(3)/2*width; 
        this.setBounds(width,height);    
                                            
        this.bottom = new zim.Rectangle(size, size, bottomColor).pos(0,0,LEFT,BOTTOM,this);
        this.top = new zim.Triangle(size, size, size, topColor).pos(0,0,CENTER,TOP,this);
        
        // ADDED for STYLE to style transforms like scale, alpha, and more
        if (style!==false) zim.styleTransforms(this, DS); 
                
    }      
    
    // or put methods out in class 
    fall(direction) {
        this.reg(LEFT,BOTTOM,true).animate({
            props:{skewX:direction==LEFT?-90:90},
            ease:"bounceOut",
            call:() => {
                // if we want to pass extra information in event object:
                const evt = new createjs.Event("fallen");
                evt.extra = "yay!";
                // this in arrow function is scoped to class object
                this.dispatchEvent(evt);
            }
        });
        return this; // for chaining
    }       
    
    dispose(a,b,disposing) {
        // remove any event functions             
        super.dispose(disposing);
    }
    
    // getter setters
    
    get bottomColor() {
        return this.bottom.color;
    }    
    set bottomColor(value) {
        this.bottom.color = zim.Pick.choose(value);
    }    
    
    get topColor() {
        return this.top.color;
    }    
    set topColor(value) {
        this.top.color = zim.Pick.choose(value);
    }        
            
}